# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import TerminaEnvironment
import httpx
from .core.client_wrapper import SyncClientWrapper
from .auth.client import AuthClient
from .group.client import GroupClient
from .user.client import UserClient
from .macro.client import MacroClient
from .company.client import CompanyClient
from .deal.client import DealClient
from .file.client import FileClient
from .financials.client import FinancialsClient
from .product.client import ProductClient
from .talent.client import TalentClient
from .unit_economics.client import UnitEconomicsClient
from .metadata.client import MetadataClient
from .benchmark.client import BenchmarkClient
from .webhook.client import WebhookClient
from .core.request_options import RequestOptions
from .types.healthz_response import HealthzResponse
from .core.pydantic_utilities import parse_obj_as
from json.decoder import JSONDecodeError
from .core.api_error import ApiError
from .core.client_wrapper import AsyncClientWrapper
from .auth.client import AsyncAuthClient
from .group.client import AsyncGroupClient
from .user.client import AsyncUserClient
from .macro.client import AsyncMacroClient
from .company.client import AsyncCompanyClient
from .deal.client import AsyncDealClient
from .file.client import AsyncFileClient
from .financials.client import AsyncFinancialsClient
from .product.client import AsyncProductClient
from .talent.client import AsyncTalentClient
from .unit_economics.client import AsyncUnitEconomicsClient
from .metadata.client import AsyncMetadataClient
from .benchmark.client import AsyncBenchmarkClient
from .webhook.client import AsyncWebhookClient


class BaseClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : TerminaEnvironment
        The environment to use for requests from the client. from .environment import TerminaEnvironment



        Defaults to TerminaEnvironment.DEFAULT



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from termina import Termina

    client = Termina(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: TerminaEnvironment = TerminaEnvironment.DEFAULT,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.auth = AuthClient(client_wrapper=self._client_wrapper)
        self.group = GroupClient(client_wrapper=self._client_wrapper)
        self.user = UserClient(client_wrapper=self._client_wrapper)
        self.macro = MacroClient(client_wrapper=self._client_wrapper)
        self.company = CompanyClient(client_wrapper=self._client_wrapper)
        self.deal = DealClient(client_wrapper=self._client_wrapper)
        self.file = FileClient(client_wrapper=self._client_wrapper)
        self.financials = FinancialsClient(client_wrapper=self._client_wrapper)
        self.product = ProductClient(client_wrapper=self._client_wrapper)
        self.talent = TalentClient(client_wrapper=self._client_wrapper)
        self.unit_economics = UnitEconomicsClient(client_wrapper=self._client_wrapper)
        self.metadata = MetadataClient(client_wrapper=self._client_wrapper)
        self.benchmark = BenchmarkClient(client_wrapper=self._client_wrapper)
        self.webhook = WebhookClient(client_wrapper=self._client_wrapper)

    def healthz(self, *, request_options: typing.Optional[RequestOptions] = None) -> HealthzResponse:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HealthzResponse
            Successful Response

        Examples
        --------
        from termina import Termina

        client = Termina(
            token="YOUR_TOKEN",
        )
        client.healthz()
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/healthz",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    HealthzResponse,
                    parse_obj_as(
                        type_=HealthzResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBaseClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : TerminaEnvironment
        The environment to use for requests from the client. from .environment import TerminaEnvironment



        Defaults to TerminaEnvironment.DEFAULT



    token : typing.Optional[typing.Union[str, typing.Callable[[], str]]]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from termina import AsyncTermina

    client = AsyncTermina(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: TerminaEnvironment = TerminaEnvironment.DEFAULT,
        token: typing.Optional[typing.Union[str, typing.Callable[[], str]]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.auth = AsyncAuthClient(client_wrapper=self._client_wrapper)
        self.group = AsyncGroupClient(client_wrapper=self._client_wrapper)
        self.user = AsyncUserClient(client_wrapper=self._client_wrapper)
        self.macro = AsyncMacroClient(client_wrapper=self._client_wrapper)
        self.company = AsyncCompanyClient(client_wrapper=self._client_wrapper)
        self.deal = AsyncDealClient(client_wrapper=self._client_wrapper)
        self.file = AsyncFileClient(client_wrapper=self._client_wrapper)
        self.financials = AsyncFinancialsClient(client_wrapper=self._client_wrapper)
        self.product = AsyncProductClient(client_wrapper=self._client_wrapper)
        self.talent = AsyncTalentClient(client_wrapper=self._client_wrapper)
        self.unit_economics = AsyncUnitEconomicsClient(client_wrapper=self._client_wrapper)
        self.metadata = AsyncMetadataClient(client_wrapper=self._client_wrapper)
        self.benchmark = AsyncBenchmarkClient(client_wrapper=self._client_wrapper)
        self.webhook = AsyncWebhookClient(client_wrapper=self._client_wrapper)

    async def healthz(self, *, request_options: typing.Optional[RequestOptions] = None) -> HealthzResponse:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HealthzResponse
            Successful Response

        Examples
        --------
        import asyncio

        from termina import AsyncTermina

        client = AsyncTermina(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.healthz()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/healthz",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    HealthzResponse,
                    parse_obj_as(
                        type_=HealthzResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: TerminaEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")
